#include "ActionTable.h"
#include "Config.h"
#include "Auxilliary.h"
#include <iostream>
#include <iomanip>

using namespace std;

extern Config config;

ActionTable::ActionTable()
{
  table = new Action*[config.numOurRobots-1];
  
  for(int i = 0; i < config.numOurRobots - 1; i++)
    {
      table[i] = new Action[config.numActions];
    }
}

ActionTable::~ActionTable()
{
	//commenting this out prevents segfault? i dont know whats goin on, but whatever - gaurav
	/*
	for(int i = 0; i < config.numOurRobots-1; i++)
		{
			cout<<"deleting ActionTable row " << i << endl;
			delete[] table[i];
		}
		*/

	delete[] table;
}

void ActionTable::calculate(Robot *robots, PotentialMap *maps, Ball *ball)
{

	double retentionWeight = 0.6;
	
	evaluateShoot(robots, maps, ball);
	evaluateClear(robots, maps, ball);
  
	/*
	//replaced by evaluateIntercept (?)
	evaluateBlock(robots, maps, ball);
	evaluateAnnoy(robots, maps, ball);
	*/

	evaluateIntercept(robots, maps, ball);
	evaluateGetOpen(robots, maps, ball);

	//to implement:
	//evaluateTrail(robots, maps, ball);

	//should somehow take into account the old action each robot was performing
	//idea: add an offset to each action the robot is already performing

	//e.g. if robot1 was planning to shoot last move, add retentionWeight to table[1][SHOOT]
	for (int i = 0; i < config.numOurRobots - 1; i++) {
		Action a = robots[i].getAction();
		table[i][a.actionID].value += retentionWeight;
	}

}

void ActionTable::evaluateShoot(Robot *robots, PotentialMap *maps, Ball *ball)
{
 
	double enemyWeight = 1;
	double orientationWeight = 2;
	
	Vector target;
	PotentialMap newMap(config.fieldDimensions, config.numCells);
	Vector endPosition, endOrientation, bestPosition, bestOrientation, bestTarget;
	int goalRow, firstTime = 1;
	double optimalValue, shotValue, totalValue, timeToShot;

	//Check which row our goal line lays on
	goalRow = (int)config.numCells.y() - 1;	//default to last row
	//Check if it should really be row 0
	for(int j = 0; j < (int)config.numCells.x(); j++) {
		if(maps[GOAL].getCell(j, 0) >= 1) goalRow = 0;
	}
  
	for(int i = 0; i < config.numOurRobots - 1; i++) {
		for(int j = 0; j < config.numCells.x(); j++) {
			if(maps[GOAL].getCell(j, goalRow) == 1) {
				target = maps[ENEMY].cellToCoord(j,goalRow);
				shotValue = (1 - maps[ENEMY].getCell(j,goalRow)) * enemyWeight;

				endOrientation = target - ball->getPosition();
				//endPosition = ball->getPosition();
				
				//endPosition is set to ball position + the distance to move "trhough" the ball --> kind of a hack of a "kick"
				endPosition = ball->getPosition() + endOrientation.normalize()*config.throughBallDistance;


				//debug printing:
				//endPosition.print("ball pos");
				/*
				if (i==1) {
					printf("shooting to: ");
					(maps[ENEMY].cellToCoord(j,goalRow)).print();
					endOrientation.print("shoot ori");
					printf("shoot ori: %lf\n", endOrientation.angle());
				}
				*/


				timeToShot = timeToMove(robots[i].getPosition(), robots[i].getOrientationVector(), endPosition, endOrientation);
				//				printf("timeToShot: %lf\n", timeToShot);

				totalValue = shotValue - timeToShot*config.timeWeight;

				Action a = robots[i].getAction();
				//orientationWeight biases shots toward points close to previous desired shot, thereby preventing drastic changes in angle
				if (a.actionID == SHOOT) {
					totalValue += orientationWeight * (1 - fabs(normalize_angle(endOrientation.angle() - a.orientation.angle())/M_PI));
				}

				if(firstTime) {
					firstTime = 0;
					optimalValue = totalValue;
					bestOrientation = endOrientation;
					bestPosition = endPosition;
					bestTarget = Vector(j,goalRow);
				}

				if(totalValue > optimalValue) {
					optimalValue = totalValue;
					bestOrientation = endOrientation;
					bestPosition = endPosition;
					bestTarget = Vector(j,goalRow);
				}
				
				/*
				//DEBUG / HACK: hardcoding shot to figure out why it veers off
				if(j == 50) {
					optimalValue = totalValue;
					bestOrientation = endOrientation;
					bestPosition = endPosition;
					bestTarget = Vector(j,goalRow);
				}
				*/
			}
			//else printf("goal row %i = %lf\n", j, maps[GOAL].getCell(0,j)); 
		}

		table[i][SHOOT] = Action(SHOOT, optimalValue * config.shootWeight, bestPosition, bestOrientation, bestTarget);
	}
}

void ActionTable::evaluateClear(Robot *robots, PotentialMap *maps, Ball *ball)
{

	double enemyWeight = 1;
	double friendWeight = 1;
	double upfieldWeight = 1;
	
	PotentialMap newMap(config.fieldDimensions, config.numCells);
	Vector endPosition, endOrientation, bestPosition, bestOrientation;
	double optimalValue, timeToClear, clearValue, totalValue;

	for(int i = 0; i < config.numOurRobots - 1; i++) {
	  for(int j = 0; j < config.numCells.x(); j++) {
	    for(int k = 0; k < config.numCells.y(); k++) {
	      clearValue = -1*maps[ENEMY].getCell(j, k)*enemyWeight + maps[UPFIELD].getCell(j, k)*upfieldWeight;

	      for(int l = 0; l < config.numOurRobots - 1; l++) {
		if(i != l)
		  {
		    //  cout << "Checking " << j << ", " << k << endl;
		  //clearValue += (robots[l].getMap()).getCell(j, k)*friendWeight;
		  clearValue += (robots[l].getMapCell(j,k))*friendWeight;
		  }
	      }
	      
	      endOrientation = maps[ENEMY].cellToCoord(j, k) - ball->getPosition();
			endPosition = ball->getPosition() + endOrientation.normalize()*config.throughBallDistance;
				
	      timeToClear = timeToMove(robots[i].getPosition(), robots[i].getOrientationVector(), endPosition, endOrientation);

				totalValue = clearValue - timeToClear*config.timeWeight;
/*
				if(firstTime) {
					firstTime = 0;
					optimalValue = totalValue;
					bestOrientation = endOrientation;
					bestPosition = endPosition;
				}
*/

				if((j==0 && k==0)||(totalValue > optimalValue)) {
					optimalValue = totalValue;
					bestOrientation = endOrientation;
					bestPosition = endPosition;
			
				}
			}
		}

		table[i][CLEAR] = 	Action(CLEAR, config.clearWeight * optimalValue, bestPosition, bestOrientation);
	}
}

void ActionTable::evaluateIntercept (Robot *robots, PotentialMap *maps, Ball *ball)
{
	
	Vector goalPos;	// ** DEFINE THIS
	Vector bestPosition, bestOrientation;
	Vector position, orientation;
	double value;
	double bestValue;
	
	for (int i = 0; i < config.numOurRobots - 1; i++) {
		//annoy = interecept enemy-ball
		for (int j = config.numOurRobots; j < NUM_ROBOTS; j++) {
			value = evaluateSpecificIntercept(robots[i], robots[j].getPosition(), ball->getPosition(), position, orientation);
			if ( (j == config.numOurRobots) || (value > bestValue) ){
				bestValue = value;
				bestPosition = position;
				bestOrientation = orientation;
			}
		}
		table[i][ANNOY] = Action(ANNOY, config.annoyWeight * bestValue, bestPosition, bestOrientation);
		
		//block = intercept ball-goal
		value = evaluateSpecificIntercept(robots[i], ball->getPosition(), goalPos, position, orientation);
		bestValue = value;
		bestPosition = position;
		bestOrientation = orientation;
		table[i][BLOCK] = Action(BLOCK, config.blockWeight * bestValue, bestPosition, bestOrientation);
	}
}

double ActionTable::evaluateSpecificIntercept (Robot robot, Vector enemyPos, Vector targetPos, Vector &bestPosition, Vector &bestOrientation) {
	
	Vector rayToIntercept, pointOnLine, rayRobotToPoint;
	
	rayToIntercept = targetPos - enemyPos;

	double numDivs = 15;
	double time1, time2, totalValue;

	double robotTimeToMoveWeight = 2;
	double enemyTimeToMoveWeight = .2;

	double bestValue;

	//for (double i = 0; i <= rayToIntercept.magnitude(); i+= divSize) {
	for (int i = 0; i <= numDivs; i++) {
		pointOnLine = enemyPos + rayToIntercept * (i/numDivs);
		rayRobotToPoint = pointOnLine - robot.getPosition();
		
		time1 = timeToMove(robot.getPosition(), robot.getOrientationVector(), pointOnLine, rayRobotToPoint);
		time2 = timeToMove(robot.getPosition(), robot.getOrientationVector(), pointOnLine, rayRobotToPoint * (-1));
		
		//totalValue takes into account time to get to target position/orientation, as well as the distance from the target to the moving enemy.
		totalValue = (robotTimeToMoveWeight * (-1) * min(time1, time2)) + (enemyTimeToMoveWeight * timeToMove(enemyPos, rayToIntercept, pointOnLine, rayToIntercept));

		if ( (i==0) || (totalValue > bestValue) ) {
			bestValue = totalValue;
			bestPosition = pointOnLine;
			if (time1 < time2) {
				bestOrientation = rayRobotToPoint;
			}
			else {
				bestOrientation = rayRobotToPoint * -1;
			}
				
		}
		
	}

	return bestValue;
	
	
}


void ActionTable::evaluateGetOpen (Robot *robots, PotentialMap *maps, Ball *ball)
{

	//double dummy, value, timeToValue, totalValue;

	double enemyWeight = 1;
	double friendWeight = 0;
	double upfieldWeight = 1;
	double centerWeight = 1;
	double timeWeight = 1;

	double totalValue, bestValue, timeToGetOpen, optimalValue;
	Vector endPosition, endOrientation;
	Vector bestPosition, bestOrientation;

	for(int i = 0; i < config.numOurRobots - 1; i++) {
		for(int j = 0; j < config.numCells.x(); j++) {
			for(int k = 0; k < config.numCells.y(); k++) {
					

			 
				endPosition = Vector(j,k);
				endOrientation = (endPosition - ball->getPosition()).normalize();
				timeToGetOpen = timeToMove(robots[i].getPosition(), robots[i].getOrientationVector(), endPosition, endOrientation);
				

				
				
				totalValue = -1*maps[ENEMY].getCell(j, k)*enemyWeight + maps[UPFIELD].getCell(j, k)*upfieldWeight + maps[CENTER].getCell(j, k)*centerWeight;

				for(int l = 0; l < config.numOurRobots - 1; l++) {
					if(i != l)
						//totalValue += robots[l].getMap().getCell(j, k)*friendWeight;
						  totalValue += (robots[l].getMapCell(j,k))*friendWeight;
				}
				totalValue = totalValue - (timeToGetOpen * config.timeWeight);




				if((j==0 && k==0)||(totalValue > optimalValue)) {
					bestValue = totalValue;
					bestPosition = endPosition;
					bestOrientation = endOrientation;
				}

			

			}
			
		
		}

	
		table[i][GETOPEN] = Action(GETOPEN, config.getopenWeight * bestValue, bestPosition, bestOrientation);
				

	}

}

void ActionTable::assignActions(Robot *robots) {
	
  int bestActions[config.numOurRobots-1];
  int numShootClear;
  double bestActionSum, actionSum;
  
  for (int i = 0; i < config.numActions; i++) {
    for (int j = 0; j < config.numActions; j++) {
      for (int k = 0; k < config.numActions; k++) {
	if (i==j || i==k || j==k) continue;	//if any two robots pick the same action, continue
	
	//following ensures clear and shoot not both attempted
	numShootClear=0;
	if (i == SHOOT || i == CLEAR) numShootClear++;
	if (j == SHOOT || j == CLEAR) numShootClear++;
	if (k == SHOOT || k == CLEAR) numShootClear++;
	if (numShootClear > 1) continue;
	
	actionSum = table[0][i].value + table[1][j].value + table[2][k].value;
	if ((i==0 && j==1 && k==2) || (actionSum > bestActionSum)) {
	  if(config.verbose)
	    printf("better actionSum [%d,%d,%d] = %lf\n", i,j,k, actionSum);
	  bestActionSum = actionSum;
	  bestActions[0] = i;
	  bestActions[1] = j;
	  bestActions[2] = k;
	}
      }
    }
    if (i == SHOOT || i == CLEAR) numShootClear++;
  }

  for(int i = 0; i < config.numOurRobots - 1; i++) {
    printf("i=%d\n",i);
    robots[i].setAction(table[i][bestActions[i]]);
  }

	//*** HACK: ***
	// set action for robots[0] and robots[1] to kick, just to test kicking skillz.
	/*
	robots[0].setAction(table[0][SHOOT]);
	robots[0].getAction().print();
	
	//DEBUG / HACK: following 2 lines set a fixed position/orientation for robot to move to, to test the move function
	table[1][SHOOT].position = Vector(500, 1000);
	table[1][SHOOT].orientation = Vector(0, 1);
	robots[1].setAction(table[1][SHOOT]);
	robots[1].getAction().print();
	*/
}

void ActionTable::print() {
	
	printf("Action Table\n");
	cout<<setw(10)<<"";
	for (int i = 0; i < config.numActions; i++) {
		cout<<setw(10)<<config.actionNames[i];
	}
	cout<<endl;
	
	for (int i = 0; i < config.numOurRobots-1; i++) {
		cout<<setw(10)<<"Robot "<<i<<":";
		for (int j = 0; j < config.numActions; j++) {
			Action a = table[i][j];
			cout<<setw(10)<<a.value;
		}
		cout<<endl;
	}
}

