#ifndef __ROBOT_H_
#define __ROBOT_H_

#include <string>
#include <math.h>

#include "Vector.h"
#include "PotentialMap.h"

#include "Action.h"

using namespace std;

class Robot
{
 public:
  Robot();
  Robot(int id);
  void init(Vector fieldDim, Vector numC);
  Vector getPosition();
  double getOrientationDouble();
  Vector getOrientationVector();
  Vector getMotors() { return motors; };
  string getName() {return name; };
  Vector getVelocity();
  PotentialMap getMap();
  double getMapCell(int x, int y);
  int getID() { return id; };
  void move( Vector endPos ); //added by spt2002
  void move( Vector endPos, Vector endOri ); //added by spt2002
  void move( Vector startPos, Vector startOri, Vector endPos, Vector endOri, double vel );
  void move( Vector startPos, Vector startOri, Vector endPos, Vector endOri, double vel, double clearDist );

  void setPosition(Vector pos);
  void setOrientation(double ori);
  void setOrientation(Vector ori);
  void setVelocity(Vector vel);
  void setMap(PotentialMap map);
  void setName(string name_);
  void setAction(Action a);
  void setID(int i) { id = i; };
  void setMotors(double mtrA, double mtrC) { 
    motors.setX(mtrA); 
    motors.setY(mtrC); 

    /* dont allow below 0.3?
    if(mtrA < 0.3){
      motors.setX(0.3);
      if(mtrA < 0.1)
	motors.setX(0.0);
      }
    if(mtrC < 0.3){
      motors.setY(0.3);
      if(mtrC < 0.1)
	motors.setY(0.0);
    }
    */
  };

  Action getAction() { return action; };
  
 private:
  Vector position;
  Vector orientation;
  Vector velocity;
  int id;  //RCX id (used for COMM signal)

  Vector motors;

  string name;
  PotentialMap map;
  Action action;
};

  
#endif
